<?php
require_once ('cms/models/vo/Content.php');
require_once ('cms/models/MyContents.php');
require_once ('user/models/vo/User.php');
require_once ('user/models/UserUtil.php');
require_once ('employee/models/Employees.php');
require_once ('event/models/Events.php');
require_once ('event/models/vo/Event.php');
require_once ('contact/models/vo/Contact.php');
require_once ('contact/models/Contacts.php');
require_once ('task/models/vo/Task.php');
require_once ('task/models/Tasks.php');
require_once ('document/models/Documents.php');
require_once ('others/class.pdf2text.php');
class Precurio_Search {
	const PATH_INDEX = "../dict";
	private $index;
	public function __construct()
	{
		if(!file_exists(self::PATH_INDEX))
		 	$this->createIndexFromDb();
		 else
		 	$this->index = Zend_Search_Lucene::open(self::PATH_INDEX);
	}
	public function optimize()
	{
		// Optimize index.
		$this->index->optimize();
	}
	private function createIndexFromDb()
	{
		set_time_limit(0);
		mkdir(self::PATH_INDEX);
		$this->index = Zend_Search_Lucene::create(self::PATH_INDEX);
		$this->indexContents();
		
		$this->indexEmployees();
		$this->indexEvents();
		$this->indexContacts();
		$this->indexTasks();
	}
	
	private function indexContents()
	{
	
		$table = new Zend_Db_Table(array('name'=>PrecurioTableConstants::CONTENT,'rowClass'=>'Content'));
		$contents = $table->fetchAll($table->select()->where('active=1'));
		foreach($contents as $content)
		{
			$this->indexContent($content,true);
		}
		
	}
	private function indexEmployees()
	{
		$table = new Zend_Db_Table(array('name'=>PrecurioTableConstants::USERS,'rowClass'=>'User'));
		$select = $table->select()->where('active=1');
		$employees  = $table->fetchAll($select);
		foreach($employees as $employee)
		{
			$this->indexEmployee($employee,true);
		}
		
	}
	private function indexEvents()
	{
		$table = new Zend_Db_Table(array('name'=>PrecurioTableConstants::EVENT, 'rowClass'=>'Event'));
		$events = $table->fetchAll($table->select()->where('active=1'));
		foreach($events as $event)
		{
			$this->indexEvent($event,true);
		}
		
	}
	private function indexContacts()
	{
		$table = new Zend_Db_Table(array('name'=>PrecurioTableConstants::CONTACTS, 'rowClass'=>'Contact'));
		$contacts = $table->fetchAll($table->select()->where('active=1'));
		foreach($contacts as $contact)
		{
			$this->indexContact($contact,true);
		}
	}
	private function indexTasks()
	{
		$table = new Zend_Db_Table(array('name'=>PrecurioTableConstants::TASK, 'rowClass'=>'Task'));
		$select = $table->select(false);
		$select->setIntegrityCheck(false);
		$select->setTable($table); 
		
		$select = $select->distinct()
						->from(array('a' => PrecurioTableConstants::TASK))
						->join(array('b' => PrecurioTableConstants::TASK_USERS),'a.id = b.task_id',array('user_id','is_proxy','is_transfer','date_assigned'=>'date_created'))
						->join(array('c' => PrecurioTableConstants::USERS),'a.creator_user_id = c.user_id',array('first_name','last_name','profile_picture_id'))
						->where('a.active=1')
						->where('b.active=1');
		$tasks = $table->fetchAll($select);
		foreach($tasks as $task)
		{
			$this->indexTask($task,true);
		}
	}
	
	/**
	 * Adds a task to the index
	 * @param $task int | Task . you could either pass the task_id or the Task object
	 * @return void
	 */
	public function indexTask($task,$indexing = false)
	{
		
		if(!is_a($task,'Task'))
			$task = Tasks::getTask($task);
		
		if(Precurio_Utils::isNull($task->title))
				return;//tasks without a title will be ignored.
		
		//check if the user already exists
		if(!$indexing)
		{
			$hits = $this->index->find('id:' . $task->id.' AND module:task');
			foreach ($hits as $hit) 
			{
				$this->index->delete($hit->id);
			}
		}
		try
		{
			$doc = new Zend_Search_Lucene_Document();
			$doc->addField(Zend_Search_Lucene_Field::keyword('id',$task->id));
			$doc->addField(Zend_Search_Lucene_Field::keyword('creator_user_id',$task->creator_user_id));
			$doc->addField(Zend_Search_Lucene_Field::keyword('user_id',$task->user_id));
			$doc->addField(Zend_Search_Lucene_Field::keyword('type',$task->type));
			$doc->addField(Zend_Search_Lucene_Field::text('title',$task->title));
			$doc->addField(Zend_Search_Lucene_Field::text('description',$task->description));
			$doc->addField(Zend_Search_Lucene_Field::text('creator',$task->getTaskCreatorName()));
			$doc->addField(Zend_Search_Lucene_Field::text('user',$task->getTaskUserName()));
			$doc->addField(Zend_Search_Lucene_Field::keyword('status',$task->status));
			$doc->addField(Zend_Search_Lucene_Field::unIndexed('due_date',$task->getDueDate()));
			$doc->addField(Zend_Search_Lucene_Field::keyword('module','task'));
		}
		catch(Exception $e)
		{
			
		}	
		
		$this->index->addDocument($doc);
		return;
	}
	/**
	 * Adds a contact to the index
	 * @param $contact int | Contact . you could either pass the contact_id or the Contact object
	 * @return void
	 */
	public function indexContact($contact,$indexing = false)
	{
		if(!is_a($contact,'Contact'))
			$contact = Contacts::getContact($contact);
		
		if(Precurio_Utils::isNull($contact))
				return;//contacts without a full name will be ignored.
		
		//check if the user already exists
		if(!$indexing)
		{
			$hits = $this->index->find('id:' . $contact->getId().' AND module:contact');
			foreach ($hits as $hit) 
			{
				$this->index->delete($hit->id);
			}
		}
		$doc = new Zend_Search_Lucene_Document();
		$doc->addField(Zend_Search_Lucene_Field::keyword('id',$contact->getId()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('user_id',$contact->user_id));
		$doc->addField(Zend_Search_Lucene_Field::text('fullname',$contact->getFullName()));
		$doc->addField(Zend_Search_Lucene_Field::text('address',$contact->getAddress()));
		$doc->addField(Zend_Search_Lucene_Field::text('company',$contact->getCompanyName()));
		$doc->addField(Zend_Search_Lucene_Field::text('email',$contact->getEmail()));
		$doc->addField(Zend_Search_Lucene_Field::text('title',$contact->getJobTitle()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('mobile',$contact->getMobilePhone()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('work',$contact->getWorkPhone()));
		$doc->addField(Zend_Search_Lucene_Field::unIndexed('shared',$contact->shared));
		$doc->addField(Zend_Search_Lucene_Field::keyword('module','contact'));
		
		
		$this->index->addDocument($doc);
		return;
	}
	/**
	 * Adds an event to the index
	 * @param $event int | Event . you could either pass the event_id or the Event object
	 * @return void
	 */
	public function indexEvent($event,$indexing = false)
	{
		if(!is_a($event,'Event'))
			$event = Events::getEvent($event);
		
		if(Precurio_Utils::isNull($event))
				return;//users without a full name will be ignored.
		
		//check if the user already exists
		if(!$indexing)
		{
			$hits = $this->index->find('id:' . $event->getId().' AND module:event');
			foreach ($hits as $hit) 
			{
				$this->index->delete($hit->id);
			}
		}
		
		//now create a new index document
		$doc = new Zend_Search_Lucene_Document();
		$doc->addField(Zend_Search_Lucene_Field::keyword('id',$event->getId()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('user_id',$event->user_id));
		$doc->addField(Zend_Search_Lucene_Field::text('title',$event->getTitle()));
		$doc->addField(Zend_Search_Lucene_Field::text('host',$event->host));
		$doc->addField(Zend_Search_Lucene_Field::text('venue',$event->venue));
		$doc->addField(Zend_Search_Lucene_Field::text('location',$event->getLocation()));
		$doc->addField(Zend_Search_Lucene_Field::text('summary',$event->getSummary()));
		$doc->addField(Zend_Search_Lucene_Field::unStored('description',$event->description));
		$doc->addField(Zend_Search_Lucene_Field::keyword('start',$event->getDate()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('time',$event->start_time));
		$doc->addField(Zend_Search_Lucene_Field::unIndexed('logo',$event->getImagePath()));
		$doc->addField(Zend_Search_Lucene_Field::unIndexed('is_past',$event->isPast()));
		$doc->addField(Zend_Search_Lucene_Field::unIndexed('is_open',$event->is_open));
		$doc->addField(Zend_Search_Lucene_Field::keyword('module','event'));
		
		$this->index->addDocument($doc);
		return;
	}	
	
	/**
	 * Adds an employee to the index
	 * @param $user int | User. you could either pass the user_id or the User object
	 * @return void
	 */
	public function indexEmployee($user,$indexing = false)
	{
		if(!is_a($user,'User'))
			$user = UserUtil::getUser($user);
		
		if(Precurio_Utils::isNull($user))
				return;//users without a full name will be ignored.
		
		//check if the user already exists
		if(!$indexing)
		{
			$hits = $this->index->find('id:' . $user->getId().' AND module:employee');
			foreach ($hits as $hit) 
			{
				$this->index->delete($hit->id);
			}
		}
		//now create a new index document
		$doc = new Zend_Search_Lucene_Document();
		$doc->addField(Zend_Search_Lucene_Field::keyword('id',$user->getId()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('title',$user->getJobTitle()));
		$doc->addField(Zend_Search_Lucene_Field::text('fullname',$user->getFullName()));
		$doc->addField(Zend_Search_Lucene_Field::text('email',$user->getEmail()));
		$doc->addField(Zend_Search_Lucene_Field::text('location',$user->getLocation()));
		$doc->addField(Zend_Search_Lucene_Field::text('department',$user->getDepartment()));
		$doc->addField(Zend_Search_Lucene_Field::text('skill',$user->getSkills()));
		$doc->addField(Zend_Search_Lucene_Field::text('description',$user->getJobDescription()));
		$doc->addField(Zend_Search_Lucene_Field::unIndexed('profile_picture',$user->getProfilePicture()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('mobile',$user->getMobilePhone()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('work',$user->getWorkPhone()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('module','employee'));
		$doc->addField(Zend_Search_Lucene_Field::keyword('tag','staff'));
		$doc->addField(Zend_Search_Lucene_Field::text('power',$user->getFullName()));
		
		$this->index->addDocument($doc);
		
		return;
	}
	/**
	 * Adds a content to the index
	 * @param $content int | Content. you could either pass the content_id or the Content object
	 * @return void
	 */
	public function indexContent($content,$indexing = false)
	{
		if(!is_a($content,'Content'))
			$content = MyContents::getContent($content);
			
		if(Precurio_Utils::isNull($content->getTitle()))
				return;//contents without title will be ignored.
				
		if(!$indexing)
		{
			//check if the content already exists
			$hits = $this->index->find('id:' . $content->getId().' AND module:cms');
			foreach ($hits as $hit) 
			{
				$this->index->delete($hit->id);
			}
		}
		//now create a new index document
		$doc = new Zend_Search_Lucene_Document();
		$doc->addField(Zend_Search_Lucene_Field::keyword('id',$content->getId()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('user_id',$content->getUserId()));
		$doc->addField(Zend_Search_Lucene_Field::text('title',$content->getTitle()));
		$doc->addField(Zend_Search_Lucene_Field::text('fullname',$content->getFullName()));
		if($content->getUrl() != '#')
			$doc->addField(Zend_Search_Lucene_Field::text('url',$content->getUrl()));
		$doc->addField(Zend_Search_Lucene_Field::text('tags',$content->getKeywords()));
		$doc->addField(Zend_Search_Lucene_Field::unIndexed('image_path',$content->getImagePath()));
		$doc->addField(Zend_Search_Lucene_Field::text('summary',$content->getSummary()));
		$doc->addField(Zend_Search_Lucene_Field::unStored('body',$content->getBody()));
		$doc->addField(Zend_Search_Lucene_Field::unIndexed('is_photo',$content->is_photo));
		$doc->addField(Zend_Search_Lucene_Field::unIndexed('is_document',$content->is_document));
		$doc->addField(Zend_Search_Lucene_Field::unIndexed('date',$content->getDateAdded()));
		$doc->addField(Zend_Search_Lucene_Field::keyword('module','cms'));
		if($content->isDocument())
		{
			$file = Documents::getDocument($content->getId());
			$root = Zend_Registry::get('root');
			$filename = $root.'/public/'.Content::PATH_TMP.$file->getFullFilename();
			file_put_contents($filename,$file->getContent());
			
			try
			{
				$file_content = "";
				if($file->isWord())
				{
					$word = new COM("word.application");
					$word->Documents->Open($filename);
					
					// Extract content.
					$content = (string) $word->ActiveDocument->Content;
					
					$file_content =  $content;
					
					$word->ActiveDocument->Close(false);
					
					$word->Quit();
					$word = null;
					unset($word);
				}
				if($file->isPdf())
				{
					$file_content = pdf2txt($filename);
					//Precurio_Utils::debug($file_content);
				}
			}
			catch(Exception $e)
			{
				
			}
			unlink($filename);//delete the file
			$doc->addField(Zend_Search_Lucene_Field::unStored('file',$file_content));
		}
		
		$this->index->addDocument($doc);
		return;
		
	}
	public function unIndexTask($task_id)
	{
		//check if the content already exists
		$hits = $this->index->find('id:' . $task_id.' AND module:task');
		foreach ($hits as $hit) 
		{
			$this->index->delete($hit->id);
		}
	}
	public function unIndexContact($contact_id)
	{
		//check if the content already exists
		$hits = $this->index->find('id:' . $contact_id.' AND module:contact');
		foreach ($hits as $hit) 
		{
			$this->index->delete($hit->id);
		}
	}
	public function unIndexEvent($event_id)
	{
		//check if the content already exists
		$hits = $this->index->find('id:' . $event_id.' AND module:event');
		foreach ($hits as $hit) 
		{
			$this->index->delete($hit->id);
		}
	}
	public function unIndexEmployee($user_id)
	{
		//check if the content already exists
		$hits = $this->index->find('id:' . $user_id.' AND module:employee');
		foreach ($hits as $hit) 
		{
			$this->index->delete($hit->id);
		}
	}
	public function unIndexContent($content_id)
	{
		//check if the content already exists
		$hits = $this->index->find('id:' . $content_id.' AND module:cms');
		foreach ($hits as $hit) 
		{
			$this->index->delete($hit->id);
		}
	}
	/**
	 * @param $queryStr
	 * @param $module
	 * @return array of Zend_Search_Lucene_Search_QueryHit objects.

	 */
	public function search($queryStr,$module = "")
	{
		$index = Zend_Search_Lucene::open(self::PATH_INDEX);
		try 
		{
			$hits = $index->find($queryStr);
		} 
		catch (Zend_Search_Lucene_Search_QueryParserException $e) 
		{
			echo "Query syntax error: " . $e->getMessage() . "\n";
			$log = Zend_Registry::get('log');
			$log->err($e);
			return;
		}
		return $hits;
		
		
	}
	
}

?>